''********************************************
''*  ADS7822P object V2.1       2007/02/14   *
''*  (C) 2007 William Henning                *
''*  http://www.mikronauts.com               *
''********************************************
''
'' This object provides an easy interface to the ADS7822P Analog to Digital converter.
''
'' The ADS7822P is a 12 bit ADC capable of 200k samples per second while consuming only
'' 1.6mW, and it is available in a nice friendly DIP-8 package. It will quite happily
'' work with a 3.3V supply so it is very Propeller friendly.  
''
'' The data sheet is available at:
''
''      http://focus.ti.com/docs/prod/folders/print/ads7822.html
''
'' The chip is VERY easy to get working; this is the assembly version of the object has
'' and provides two methods.
''
''      Start(@longvar, basepin, bits)
''
''              @longvar        Address of a 'LONG' spin variable where the conversion
''                              results will be stored         
''
''              basepin         DATA line of the A/D converter
''                              basepin+1       CLOCK line
''                              basepin+2       CHIP_SELECT for the A/D converter
''
''              bits            number of bits you want - ie 9 for 9 bit conversion,
''                              12 for 12 bit conversion. Valid range: 6-12
''
''      Stop
''              Stops the cog that was Start()ed
''
''
'' PLEASE NOTE:
''
''      V2.1
''
''      The higher voltage version of the ADS7822P is spec'ed for 200Ksps; this version
''      of the cog driver is timed for 200Ksps sample rate with a 3MHz clock. This may
''      not work perfectly at 3.3V - do not be surprised if there is a bit more conversion
''      error than at 75Ksps
''
''      This version samples at the rates shown below for the resolution specified:
''
''         333,333 samples/sec @ 6 bits
''         294,118 samples/sec @ 7 bits
''         277,777 samples/sec @ 8 bits
''         250,000 samples/sec @ 9 bits
''         238,095 samples/sec @ 10 bits 
''         222,000 samples/sec @ 11 bits 
''         200,000 samples/sec @ 12 bits
''
''      V2.0
''
''      The ADS7822P A/D converter is specified as having a 75ksps limit, and it is rated
''      for a maximum of 1.25MHz clock. The assembly routines exactly meet the timing
''      requirements as per the data sheet.
''
''      As an experiment, I tried running the converter faster, and I was able to clock it
''      slightly in excess of 2.5MHz, resulting in over 150ksps - however the data appeared
''      to be noisier, and as such I do not suggest running the converter overspec.
''
''      If you want to run somewhat faster, the data sheet suggests that conversions can be
''      cut off early, so if you ask for eight bit conversions, you will actually get 100ksps
''      while still meeting the data sheet timing requirements!
''
''
'' I will be maintaining this object on my mikronauts.com site.
''
'' The next version will support 8 input channels via an external multiplexer
''
'' Everyone, especually Parallax, may use this object as they see fit.
''
'' William Henning
'' http://www.mikronauts.com
''
VAR
    word mycog

PUB Start(aPtr, pin, wbits)
    a_dat := 1 << pin
    a_clk := a_dat << 1
    a_cs  := a_clk << 1
    if (wbits < 6)
      wbits := 6
    if (wbits > 12)
      wbits := 12
    wantbits := wbits
    mycog := cognew(@a2d_cog, aPtr)

PUB Stop
    cogstop(mycog)

DAT

        ' 80 + 9*24 = 296 clock cycles + 16 for the wrlong&jump = 312 cycles for 9 bits = 250ksps after hub delay
        ' 304 cycles between hub access alignments, + 16 for writelong&jmp = 320 cycles for 9 bits

        ' 80 + 6*24 = 224 + 16 = 240 = 3us = 333,333 samples/sec @ 6 bits
        ' 80 + 7*24 = 248 + 16 + 8 hub delay = 294,118 samples/sec @ 7 bits
        ' 80 + 8*24 = 272 + 16 = 288 = 3.6us = 277,777 samples/sec @ 8 bits
        ' 80 + 9*24 = 296 + 16 = 312 + 8 hub delay = 4us = 250,000 samples/sec @ 9 bits
        ' 80 + 10*24 = 320 + 16 = 336 / 80 = 4.2us = 238,095 samples/sec @ 10 bits 
        ' 80 + 11*24 = 344 + 16 = 360 / 80 = 4.5us = 222,000 samples/sec @ 10 bits 
        ' 80 + 12*24 = 384 + 16 = 400 / 80 = 5us = 200,000 samples/sec @ 12 bits

        org   0
        
a2d_cog ' set up data direction register and cobined CS & CLK mask

        mov   t,a_cs
        or    t,a_clk
        mov   a_cs_clk,t ' combined bitmask for CS and CLK
        mov   dira,t     ' enable io bits as output

        ' start the conversion

more    mov  outa,#0    
        nop
        nop
        mov   outa,a_clk 
        nop
        nop

        mov  outa,#0    
        nop
        nop
        mov   outa,a_clk 
        nop
        nop

        mov  outa,#0    
        nop
        mov   adcval,#0
        mov   outa,a_clk 
        nop
        mov   adcbits,wantbits         ' re-doing instead of nops

        ' get the conversion result (18*4)
        
getbit  mov   outa,#0           ' we cheat a bit, low period will be half of high period   
        nop

        mov   outa,a_clk 
        test  a_dat,ina wc
        addx  adcval,adcval
        djnz  adcbits,#getbit

        ' de-select A/D converter (18*4 + adcbits*6*4 + 1*4) cycles so far
        
        mov   outa,a_cs_clk

        ' update global with conversion result (20*4 + adcbits*24) so far         
        
        wrlong adcval, par

end     jmp   #more

loops    long  3
adcbits  long  12
wantbits long  12
adcval   long  0
t        long  0
a_dat    long  0
a_clk    long  0
a_cs     long  0
a_cs_clk long  0                 